// I2C_Sample.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <windows.h>
#include <commctrl.h>
#include <winioctl.h>
#include "i2c.h"

// SAA7113H address on SB-X270
#define SLAVE_ADDRESS1 0x25
// SAA7113H address on ATX 
#define SLAVE_ADDRESS2 0x24

// Chip ID
#define REGISTER_NUMBER1 0x0
// Writable register
#define REGISTER_NUMBER2 0x41

#define DATA1	0x1f
#define DATA2	0xf1

/**********************************************************************************************************
 About I2C_Sample.
***********************************************************************************************************
 This sample demonstates the usage of I2C bus from user application.
 The I2C driver that comes with CM_X270_BSP since 17/09/2007 allows to access
 more than one device on I2C bus.
 The I2C device can be accessed via I2CTRANS transaction structure.
 See the demonstration of I2C transaction under the CUSTOM_TRANSACTION_DEMO defne

 To evaluate the I2C driver on CM-X270L on SB-X270 (or ATX Base) the example below is written.
 The main scenario is to acces the codec with SLAVE_ADDRESS1 and SLAVE_ADDRESS2.
 If the codec with the SLAVE_ADDRESS2 is not present (on SB-X270) the resulted read value 
 will be 0.
 To validate that the codec is responding the following scenario is used:
	- read temp (writable) register
	- write temp register with some value
	- read temp register again to see the changes
The I2CReadByte and I2CWriteByte (defined in i2c.h) demonstrate the byte-wide access to the I2C device
and are used for SAA7113H codec demonstration.

WARNING:
	Before testing I2C bus with SAA7113H codec, ensure that the codec is enabled:
		On SB-X270 the E5 jumper should be removed.
		On ATX the pin 17 and pin 18 of P38 should be shorten (also remove the jumper P18<->P19 if exists). 

************************************************************************************************************/

int _tmain(int argc, _TCHAR* argv[])
{
	UCHAR ucSlaveAddr;
	UCHAR ucRegisterNumber;
	UCHAR ucData=0;

	//Open a handle to I2C
	HANDLE hI2C = I2CUserOpen(I2C_BUS_NAME);
	if(hI2C == INVALID_HANDLE_VALUE)
	{
		printf("Error opening I2C handle\n");
		return 0;
	}
	
	//Choose slave address
	//The PXA-270 only supports 7 bit slave address so make sure 
	//the address you have chosen isn't greater than 127...
	printf(" Setting Slave address as fits on the SB-X270 board\r\n");
	ucSlaveAddr = SLAVE_ADDRESS1;
	
	//A simple read transaction 
	// Read SAA7113H codec ID
	ucRegisterNumber = REGISTER_NUMBER1;
	ucData=0;
	I2CReadByte(hI2C, ucSlaveAddr, ucRegisterNumber, &ucData);
	printf("Read from 7-bit address 0x%x reg = 0x%x at value = 0x%x\n",ucSlaveAddr, ucRegisterNumber, ucData);

	// Read SAA7113H writable temp register
	ucRegisterNumber = REGISTER_NUMBER2;
	ucData=0;
	I2CReadByte(hI2C, ucSlaveAddr, ucRegisterNumber, &ucData);
	printf("Read from 7-bit address 0x%x reg = 0x%x at value = 0x%x\n",ucSlaveAddr, ucRegisterNumber, ucData);

	//This time we will write a byte to another register
	ucData = DATA1;
	ucRegisterNumber = REGISTER_NUMBER2;
	I2CWriteByte(hI2C, ucSlaveAddr, ucRegisterNumber, ucData);
	printf("Write to 7-bit address 0x%x at reg = 0x%x at value = 0x%x\n",ucSlaveAddr, ucRegisterNumber, ucData);

	// Read SAA7113H writable temp register
	ucRegisterNumber = REGISTER_NUMBER2;
	ucData=0;
	I2CReadByte(hI2C, ucSlaveAddr, ucRegisterNumber, &ucData);
	printf("Read from 7-bit address 0x%x reg = 0x%x at value = 0x%x\n",ucSlaveAddr, ucRegisterNumber, ucData);

	
	//Choose another slave address (possibly not connected)
	printf(" Setting Slave address as fits on the ATX board\r\n");
	//Reset data;
	ucData = 0;
	ucSlaveAddr = SLAVE_ADDRESS2;

	// Read SAA7113H codec ID
	ucRegisterNumber = REGISTER_NUMBER1;
	ucData=0;
	I2CReadByte(hI2C, ucSlaveAddr, ucRegisterNumber, &ucData);
	printf("Read from 7-bit address 0x%x reg = 0x%x at value = 0x%x\n",ucSlaveAddr, ucRegisterNumber, ucData);

	// Read SAA7113H writable temp register
	ucRegisterNumber = REGISTER_NUMBER2;
	ucData=0;
	I2CReadByte(hI2C, ucSlaveAddr, ucRegisterNumber, &ucData);
	printf("Read from 7-bit address 0x%x reg = 0x%x at value = 0x%x\n",ucSlaveAddr, ucRegisterNumber, ucData);

	//This time we will write a byte to another register
	ucData = DATA2;
	ucRegisterNumber = REGISTER_NUMBER2;
	I2CWriteByte(hI2C, ucSlaveAddr, ucRegisterNumber, ucData);
	printf("Write to 7-bit address 0x%x at reg = 0x%x at value = 0x%x\n",ucSlaveAddr, ucRegisterNumber, ucData);

	// Read SAA7113H writable temp register
	ucRegisterNumber = REGISTER_NUMBER2;
	ucData=0;
	I2CReadByte(hI2C, ucSlaveAddr, ucRegisterNumber, &ucData);
	printf("Read from 7-bit address 0x%x reg = 0x%x at value = 0x%x\n",ucSlaveAddr, ucRegisterNumber, ucData);
	

	//A custom transaction
	//This mechanism allows you to implement complex protocols easily

	//Suppose your custom transaction is made up of 2 phases:
	//in the first phase you write 3 bytes and in the second you read 19...
	

	//Setup the transfer
#ifdef CUSTOM_TRANSACTION_DEMO
	UCHAR FirstTransfer[3] = {0xA, 0xB, 0xC};
	UCHAR SecondTransfer[19];
	I2CTRANS trans;
	DWORD	pdwOpCode[I2CTRANS_MAX_STREAMED_TRANSACTIONS];
	DWORD	pdwTransLen[I2CTRANS_MAX_STREAMED_TRANSACTIONS];
	DWORD	pdwBufOffset[I2CTRANS_MAX_STREAMED_TRANSACTIONS];
	BYTE	pbBuffer[I2CTRANS_BUFFER_BYTES];
	BOOL res;

	memset(&trans,0, sizeof(I2CTRANS));
	
	//Init buffers
	trans.mBuffer = pbBuffer;
	trans.mBufferOffset = pdwBufOffset;
	trans.mOpCode = pdwOpCode;
	trans.mTransLen = pdwTransLen;

	//specify clock speed (400KHz is also possible) and slave address
	//for this entire transaction
	trans.mClkSpeed = I2C_CLOCK_100Khz; 
	trans.mDeviceAddr = SLAVE_ADDRESS1; 
	trans.mTransactions = 2;			//2 transfers


	//define the first transfer (index 0)
	trans.mOpCode[0] = I2C_OPCODE_WRITE;	//define the first transfer as write
	trans.mTransLen[0] = 3;					//3 bytes 
	trans.mBufferOffset[0] = 0;				//this transfer offset in the buffer
	//fill the buffer with the data for this transfer
	memcpy(trans.mBuffer, FirstTransfer, 3);	
	
	//define the second transfer
	trans.mOpCode[1]= I2C_OPCODE_READ;
	trans.mTransLen[1] = 19;		//19 bytes will be read from the device
	trans.mBufferOffset[1] = 3;
	
	//This will start the transaction
	// = CheckTransactionIsValid(&trans);
	res = I2CUserTransact(hI2C, &trans);
	if(!res)
	{
		printf("The transaction failed\n");
		return 0;
	}
	memcpy(SecondTransfer, trans.mBuffer+3, 19);
	//SecondeTransfer buffer now stores the data trasmitted from the device
#endif

	//Closes the handle
	I2CUserClose(hI2C);
	return 0;
}

