//------------------------------------------------------------------------------
//
//  File:  i2c.h
//
#ifndef __I2C_H
#define __I2C_H


extern "C"
{
	BOOL KernelIoControl(DWORD dwIoControlCode, LPVOID lpInBuf, DWORD nInBufSize, LPVOID lpOutBuf, DWORD nOutBufSize, LPDWORD lpBytesReturned);
}
//
// IOCTL info, needs to be visible for application
//
#define I2C_IOCTL_INDEX								0x0820


#define IOCTL_I2C_TRANSACT								CTL_CODE( FILE_DEVICE_UNKNOWN,  \
                                                        I2C_IOCTL_INDEX + 0,       \
                                                        METHOD_BUFFERED,                \
                                                        FILE_ANY_ACCESS)


#define IOCTL_I2C_GET_DRIVER_IFC						CTL_CODE( FILE_DEVICE_UNKNOWN,  \
                                                        I2C_IOCTL_INDEX + 1,       \
                                                        METHOD_BUFFERED,                \
                                                        FILE_ANY_ACCESS)


//------------------------------------------------------------------------------
//
//  STRUCT: I2CTRANS
//
#define I2C_OPCODE_READ				1
#define I2C_OPCODE_WRITE			2

#define I2C_CLOCK_REGULAR_MODE		1
#define I2C_CLOCK_FAST_MODE			2

#define I2C_CLOCK_100Khz			I2C_CLOCK_REGULAR_MODE 
#define I2C_CLOCK_400Khz			I2C_CLOCK_FAST_MODE 
#define POWER_I2C_CLOCK_40KHZ		I2C_CLOCK_REGULAR_MODE
#define POWER_I2C_CLOCK_160KHZ		I2C_CLOCK_FAST_MODE

#define I2C_CLOCK_DEFAULT			I2C_CLOCK_REGULAR_MODE

#define POWER_I2C_DEVICE_ARRAY_INDEX	2
#define POWER_I2C_BUS_NAME				L"I2C2:"

#define I2C_DEVICE_ARRAY_INDEX			1
#define I2C_BUS_NAME					L"I2C1:"

#define I2CTRANS_MAX_STREAMED_TRANSACTIONS  8
#define I2CTRANS_BUFFER_BYTES               256

/*
	This is an example which will explain how to set the following struct:
	Let us assume you have a device which need to perform the following transaction 
	(all operations refer to the same register)-
		Write 3 bytes, Read 5 bytes, Write 1 byte
	mTransactions - As we can see we have 3 operations here, thus we will set 
		mTransactions=3
	mClkSpeed - is self explanatory and can use a default value
	mDeviceAddr - is self explanatory as well - just remember to use an address of no more then 7 bits
	mOpCode - In this example we have 3 operations (write, read, write) so we will set
			  mOpCode[0]=I2C_OPCODE_WRITE;mOpCode[1]=I2C_OPCODE_READ;mOpCode[2]=I2C_OPCODE_WRITE;
	mBufferOffset - For all 3 operations we have only one buffer, and so we need to specify to each operation
					where in the buffer they need to look, and so the setting will be as follows:
						-mBufferOffset[0]=0; //Because the first operations data is at the start of the buffer
						-mBufferOffset[1]=3; //Because we used bytes 0,1,2 for the first operation
						-mBufferOffset[2]=8; //Because we used bytes 0,1,2 for the 1st op and 3,4,5,6,7 for the 2nd op
	mTransLen - Specifies how many bytes from the buffer, starting at the offset, the operation will "consume":
						-mTransLen[0]=3;
						-mTransLen[1]=5;
						-mTransLen[2]=1;

	That's it.
*/
typedef struct _i2cTransaction
{
    /* input: */
	DWORD	mTransactions;
	DWORD	mClkSpeed;
	UCHAR	mDeviceAddr;  //7 bit
	DWORD	*mOpCode;
    DWORD	*mBufferOffset;
    DWORD	*mTransLen;
    /* output: */
    DWORD	mErrorCode;
    UCHAR	*mBuffer;
} I2CTRANS;



//------------------------------------------------------------------------------
//
//  Type:  DEVICE_IFC_I2C
//
//  This structure is used to obtain I2C interface funtion pointers used for
//  in-process calls via IOCTL_I2C_GET_DRIVER_IFC.
//
typedef struct {
	DWORD context;
    BOOL (*pfnTransact)(DWORD context, I2CTRANS *pTrans);
} DEVICE_IFC_I2C;



//------------------------------------------------------------------------------
//
//  Functions: I2Cxxx
//------------------------------------------------------------------------------
//
//  Functions: I2Cxxx for accessing I2C from user space via IOCTLs
//------------------------------------------------------------------------------

__inline HANDLE I2CUserOpen(TCHAR *strName)
{
    HANDLE hDevice;
	hDevice = CreateFile (strName, 0, 0, NULL, OPEN_ALWAYS | CREATE_NEW, 0, NULL);
    return hDevice;
}

__inline void I2CUserClose(HANDLE hContext)
{
	if(hContext == NULL)
		return;
	CloseHandle(hContext);
}
__inline BOOL I2CUserTransact(HANDLE hDevice, I2CTRANS* pTrans)
{
	BOOL iRet;
	DWORD BytesWritten;
	if(hDevice == NULL)
		return FALSE;
	iRet = DeviceIoControl (hDevice, IOCTL_I2C_TRANSACT, NULL, 0, pTrans, sizeof(I2CTRANS), &BytesWritten, NULL);
	return (iRet && BytesWritten == sizeof(I2CTRANS));
}


__inline BOOL I2CReadByte(HANDLE hDevice, UCHAR ucSlaveAddr, UCHAR ucRegisterNumber, BYTE *data)
{
	BOOL bRet;
	DWORD	pdwOpCode[I2CTRANS_MAX_STREAMED_TRANSACTIONS];
	DWORD	pdwTransLen[I2CTRANS_MAX_STREAMED_TRANSACTIONS];
	DWORD	pdwBufOffset[I2CTRANS_MAX_STREAMED_TRANSACTIONS];
	BYTE	pbBuffer[I2CTRANS_BUFFER_BYTES];
	if(hDevice == NULL)
		return FALSE;
	I2CTRANS trans;
	memset(&trans,0,sizeof(I2CTRANS));

	//Init buffers
	trans.mBuffer = pbBuffer;
	trans.mBufferOffset = pdwBufOffset;
	trans.mOpCode = pdwOpCode;
	trans.mTransLen = pdwTransLen;

	trans.mTransactions = 2;
	trans.mDeviceAddr = ucSlaveAddr;
	trans.mClkSpeed = I2C_CLOCK_FAST_MODE;
	//first transaction choose register
	trans.mOpCode[0] = I2C_OPCODE_WRITE;
	trans.mTransLen[0] = 1;
	trans.mBuffer[0] = ucRegisterNumber;	
	trans.mBufferOffset[0] = 0;
	//second transaction - read the content of the register
	trans.mOpCode[1] = I2C_OPCODE_READ;
	trans.mTransLen[1] = 1;
	trans.mBufferOffset[1] = 1;
	bRet = I2CUserTransact(hDevice, &trans);
	*data = trans.mBuffer[1];
	return bRet;	
}

__inline BOOL I2CWriteByte(HANDLE hDevice, UCHAR ucSlaveAddr, UCHAR ucRegisterNumber, BYTE data)
{
	BOOL bRet;
	DWORD	pdwOpCode[I2CTRANS_MAX_STREAMED_TRANSACTIONS];
	DWORD	pdwTransLen[I2CTRANS_MAX_STREAMED_TRANSACTIONS];
	DWORD	pdwBufOffset[I2CTRANS_MAX_STREAMED_TRANSACTIONS];
	BYTE	pbBuffer[I2CTRANS_BUFFER_BYTES];

	if(hDevice == NULL)
		return FALSE;
	I2CTRANS trans;
	memset(&trans,0,sizeof(I2CTRANS));
	//Init buffers
	trans.mBuffer = pbBuffer;
	trans.mBufferOffset = pdwBufOffset;
	trans.mOpCode = pdwOpCode;
	trans.mTransLen = pdwTransLen;

	trans.mTransactions = 1;
	trans.mDeviceAddr = ucSlaveAddr;
	trans.mClkSpeed = I2C_CLOCK_FAST_MODE;
	//first transaction choose register
	trans.mOpCode[0] = I2C_OPCODE_WRITE;
	trans.mTransLen[0] = 2;
	trans.mBufferOffset[0] = 0;
	
	trans.mBuffer[0] = ucRegisterNumber;	
	trans.mBuffer[1] = data;
	bRet = I2CUserTransact(hDevice, &trans);
	return bRet;	
}


#endif
